import collections

import numpy as np
import PIL.Image
import PIL.ImageDraw
import PIL.ImageFont

from .. import utils


def triangle(src, center, size, fill=None, outline=None):
    """Draw triangle on numpy array with Pillow.

    Parameters
    ----------
    src: numpy.ndarray
        Input image.
    center: (2,) array-like
        center is (cy, cx).
    size: float
        Diameter to create the star.
    fill: int or (3,) array-like, optional
        RGB color to fill the mark. None for no fill. (default: None)
    outline: int or (3,) array-like, optional
        RGB color to draw the outline.

    Returns
    -------
    dst: numpy.ndarray
        Output image.

    """
    dst = utils.numpy_to_pillow(src)
    triangle_(img=dst, center=center, size=size, fill=fill, outline=outline)
    return utils.pillow_to_numpy(dst)


def triangle_(img, center, size, fill=None, outline=None):
    if isinstance(fill, collections.abc.Iterable):
        fill = tuple(fill)
    if isinstance(outline, collections.abc.Iterable):
        outline = tuple(outline)

    draw = PIL.ImageDraw.Draw(img)

    radius = size / 2
    cy, cx = center

    x = cx + radius * np.cos(np.deg2rad(np.arange(0, 3) * 120 + 90))
    y = cy - radius * np.sin(np.deg2rad(np.arange(0, 3) * 120 + 90))

    xy = np.stack((x, y), axis=1)
    xy = xy.flatten().tolist()
    draw.polygon(xy, fill=fill, outline=outline)



import numpy as np
import PIL.Image


def pillow_to_numpy(img):
    img_numpy = np.asarray(img)
    if not img_numpy.flags.writeable:
        img_numpy = np.array(img)
    return img_numpy


def numpy_to_pillow(img, mode=None):
    img_pillow = PIL.Image.fromarray(img, mode=mode)
    return img_pillow



import collections

import numpy as np
import PIL.Image
import PIL.ImageDraw
import PIL.ImageFont

from .. import utils


def star(src, center, size, fill=None, outline=None):
    """Draw star on numpy array with Pillow.

    Parameters
    ----------
    src: numpy.ndarray
        Input image.
    center: (2,) array-like
        center is (cy, cx).
    size: float
        Diameter to create the star.
    fill: int or (3,) array-like, optional
        RGB color to fill the mark. None for no fill. (default: None)
    outline: int or (3,) array-like, optional
        RGB color to draw the outline.

    Returns
    -------
    dst: numpy.ndarray
        Output image.

    """
    dst = utils.numpy_to_pillow(src)
    star_(
        img=dst,
        center=center,
        size=size,
        fill=fill,
        outline=outline,
    )
    return utils.pillow_to_numpy(dst)


def star_(img, center, size, fill=None, outline=None):
    if isinstance(fill, collections.abc.Iterable):
        fill = tuple(fill)
    if isinstance(outline, collections.abc.Iterable):
        outline = tuple(outline)

    draw = PIL.ImageDraw.Draw(img)

    radius = size / 2
    cy, cx = center

    # 5 mountains
    angles_m = np.arange(0, 5) * np.pi * 2 / 5 + (np.pi / 2)
    x_m = cx + radius * np.cos(angles_m)
    y_m = cy - radius * np.sin(angles_m)
    xy_m = np.stack((x_m, y_m), axis=1)

    # 5 valleys
    angles_v = angles_m + np.pi / 5
    length = radius / (np.sin(np.pi / 5) / np.tan(np.pi / 10) + np.cos(np.pi / 10))
    x_v = cx + length * np.cos(angles_v)
    y_v = cy - length * np.sin(angles_v)
    xy_v = np.stack((x_v, y_v), axis=1)

    xy = np.array(
        [
            xy_m[0],
            xy_v[0],
            xy_m[1],
            xy_v[1],
            xy_m[2],
            xy_v[2],
            xy_m[3],
            xy_v[3],
            xy_m[4],
            xy_v[4],
            xy_m[0],
        ]
    )
    xy = xy.flatten().tolist()
    draw.polygon(xy, fill=fill, outline=outline)



from __future__ import division
from __future__ import print_function

import sys
import types

import numpy as np
import PIL.Image

from ... import utils
from .base import check_pyglet_available


def pyglet_imshow(image, caption=None, interval=0.5, keymap=None, hook=None):
    """Show image with pyglet.

    Parameters
    ----------
    image: numpy.ndarray or list of numpy.ndarray or iterator of numpy.array
        Image or images to show.
    caption: str, optional
        Caption for pyglet window.
    interval: float, optional
        Interval for list or iterator of images. Default is 0.5.
    keymap: dict, optional
        Key mappings for key and function.
    hook: callable, optional
        Hook function called after each image update.

    Returns
    -------
    None

    """
    if isinstance(image, types.GeneratorType):
        _pyglet_imshow_generator(
            image,
            caption=caption,
            interval=interval,
            keymap=keymap,
            hook=hook,
        )
    elif isinstance(image, np.ndarray):
        _pyglet_imshow_ndarray(image, caption=caption, keymap=keymap, hook=hook)
    else:
        _pyglet_imshow_list(
            image,
            caption=caption,
            interval=interval,
            keymap=keymap,
            hook=hook,
        )


def _pyglet_imshow_list(images, caption=None, interval=0.5, keymap=None, hook=None):
    pyglet = check_pyglet_available()

    def usage():
        print("Usage: ", file=sys.stderr)
        if keymap is not None:
            for key, function in keymap.items():
                print("  {}: {}".format(key, function))
        print("  h: show help", file=sys.stderr)
        print("  q: close window", file=sys.stderr)
        print("  n: next image", file=sys.stderr)
        print("  p: previous image", file=sys.stderr)
        print("  s: toggle play", file=sys.stderr)

    usage()

    if isinstance(images[0], np.ndarray):
        max_image_height, max_image_width = np.max(
            [image.shape[:2] for image in images], axis=0
        )
    elif isinstance(images[0], PIL.Image.Image):
        max_image_width, max_image_height = np.max(
            [image.size for image in images], axis=0
        )
    elif hook is None:
        raise ValueError(
            "hook must be specified when images is not list of "
            "numpy.ndarray or PIL.Image.Image"
        )
    else:
        image = hook(images[0])
        if not isinstance(image, (np.ndarray, PIL.Image.Image)):
            raise TypeError("hook must return numpy.ndarray or PIL.Image.Image")
        max_image_height, max_image_width = image.shape[:2]
    aspect_ratio = max_image_width / max_image_height

    index = 0

    window = _initialize_window(caption=caption, aspect_ratio=aspect_ratio)
    window.index = index
    window.play = False

    image = _convert_to_imagedata(
        hook(images[window.index]) if hook else images[window.index]
    )
    sprite = pyglet.sprite.Sprite(image)

    def _post_image_update():
        _centerize_sprite_in_window(sprite, window)
        window.set_caption(
            "{} {}/{}".format(images[window.index], window.index + 1, len(images))
        )
        print(
            "{} {}/{}".format(images[window.index], window.index + 1, len(images)),
            file=sys.stderr,
        )

    _post_image_update()

    @window.event
    def on_draw():
        window.clear()
        sprite.draw()

    def play_callback(dt):
        if window.play:
            if window.index == len(images) - 1:
                print("Press 'q' to quit", file=sys.stderr)
                window.play = False
            window.index = min(window.index + 1, len(images) - 1)
            sprite.image = _convert_to_imagedata(
                hook(images[window.index]) if hook else images[window.index]
            )
            _centerize_sprite_in_window(sprite, window)

    pyglet.clock.schedule_interval(play_callback, interval)

    @window.event
    def on_key_press(symbol, modifiers):
        if keymap is not None:
            for key, function in keymap.items():
                if symbol == getattr(pyglet.window.key, key.upper()):
                    function(images, window.index)
                    return
        if symbol == pyglet.window.key.Q:
            window.close()
        elif symbol == pyglet.window.key.N:
            if window.index + 1 <= len(images) - 1:
                window.index += 1
                sprite.image = _convert_to_imagedata(
                    hook(images[window.index]) if hook else images[window.index]
                )
                _post_image_update()
        elif symbol == pyglet.window.key.P:
            if window.index - 1 >= 0:
                window.index -= 1
                sprite.image = _convert_to_imagedata(
                    hook(images[window.index]) if hook else images[window.index]
                )
                _post_image_update()
        elif symbol == pyglet.window.key.S:
            window.play = not window.play
        elif symbol == pyglet.window.key.H:
            usage()
        else:
            print("Press 'h' to show help", file=sys.stderr)


def _pyglet_imshow_generator(
    images, caption=None, interval=0.5, keymap=None, hook=None
):
    if keymap is not None:
        raise NotImplementedError

    pyglet = check_pyglet_available()

    image = hook(next(images)) if hook else images
    if not isinstance(image, (np.ndarray, PIL.Image.Image)):
        raise TypeError("hook must return numpy.ndarray or PIL.Image.Image")

    aspect_ratio = image.shape[1] / image.shape[0]  # width / height
    window = _initialize_window(caption=caption, aspect_ratio=aspect_ratio)

    image = _convert_to_imagedata(image)
    sprite = pyglet.sprite.Sprite(image)
    _centerize_sprite_in_window(sprite, window)

    window.play = False

    @window.event
    def on_draw():
        window.clear()
        sprite.draw()

    def usage():
        print("Usage: ", file=sys.stderr)
        print("  h: show help", file=sys.stderr)
        print("  q: close window", file=sys.stderr)
        print("  n: next image", file=sys.stderr)
        print("  p: previous image", file=sys.stderr)
        print("  s: toggle play", file=sys.stderr)

    def play_callback(dt):
        if window.play:
            try:
                sprite.image = _convert_to_imagedata(next(images))
                _centerize_sprite_in_window(sprite, window)
            except StopIteration:
                print("Press 'q' to quit", file=sys.stderr)
                window.play = False

    pyglet.clock.schedule_interval(play_callback, interval)

    @window.event()
    def on_key_press(symbol, modifiers):
        if symbol == pyglet.window.key.Q:
            window.close()
        elif symbol == pyglet.window.key.N:
            try:
                sprite.image = _convert_to_imagedata(
                    hook(next(images)) if hook else next(images)
                )
                _centerize_sprite_in_window(sprite, window)
            except StopIteration:
                print("Press 'q' to quit", file=sys.stderr)
        elif symbol == pyglet.window.key.S:
            window.play = not window.play
        elif symbol == pyglet.window.key.H:
            usage()
        else:
            print("Press 'h' to show help", file=sys.stderr)


def _pyglet_imshow_ndarray(image, caption=None, keymap=None, hook=None):
    if keymap is not None:
        raise NotImplementedError

    pyglet = check_pyglet_available()

    image = hook(image) if hook else image
    if not isinstance(image, (np.ndarray, PIL.Image.Image)):
        raise TypeError("hook must return numpy.ndarray or PIL.Image.Image")
    aspect_ratio = image.shape[1] / image.shape[0]  # width / height
    window = _initialize_window(caption=caption, aspect_ratio=aspect_ratio)

    image = _convert_to_imagedata(image)
    sprite = pyglet.sprite.Sprite(image)
    _centerize_sprite_in_window(sprite, window)

    @window.event
    def on_draw():
        window.clear()
        sprite.draw()

    def usage():
        print("Usage: ", file=sys.stderr)
        print("  h: show help", file=sys.stderr)
        print("  q: close window", file=sys.stderr)

    @window.event()
    def on_key_press(symbol, modifiers):
        if symbol == pyglet.window.key.Q:
            window.close()
        elif symbol == pyglet.window.key.H:
            usage()
        else:
            print("Press 'h' to show help", file=sys.stderr)


def _initialize_window(caption, aspect_ratio):
    pyglet = check_pyglet_available()

    display = pyglet.canvas.Display()
    screen = display.get_default_screen()

    max_window_width = int(round(screen.width * 0.75))
    max_window_height = int(round(screen.height * 0.75))

    # try to fit the image into the screen
    if aspect_ratio > 1:  # width > height
        window_width = max_window_width
        window_height = int(round(window_width / aspect_ratio))
    else:
        window_height = max_window_height
        window_width = int(round(window_height * aspect_ratio))

    # if still too large, shrink it
    if window_width > max_window_width:
        window_width = max_window_width
        window_height = int(round(window_width / aspect_ratio))
    if window_height > max_window_height:
        window_height = max_window_height
        window_width = int(round(window_height * aspect_ratio))

    window = pyglet.window.Window(
        width=window_width,
        height=window_height,
        caption=caption,
    )
    return window


def _centerize_sprite_in_window(sprite, window):
    scale_x = 0.95 * window.width / sprite.image.width
    scale_y = 0.95 * window.height / sprite.image.height
    scale = min(scale_x, scale_y)

    width = sprite.image.width * scale
    height = sprite.image.height * scale
    x = (window.width - width) / 2.0
    y = (window.height - height) / 2.0

    sprite.update(x=x, y=y, scale=scale)


def _convert_to_imagedata(image):
    pyglet = check_pyglet_available()

    if isinstance(image, np.ndarray):
        image = utils.numpy_to_pillow(image)
    elif isinstance(image, PIL.Image.Image):
        pass
    else:
        raise ValueError

    image = image.convert("RGB")

    kwargs = dict(
        width=image.width,
        height=image.height,
        data=image.tobytes(),
        pitch=-image.width * len(image.mode),
    )
    if hasattr(pyglet, "__version__") and pyglet.__version__[0] == "2":
        kwargs["fmt"] = image.mode
    else:
        kwargs["format"] = image.mode
    image = pyglet.image.ImageData(**kwargs)
    return image



import typing  # NOQA

import matplotlib
import numpy as np

from .normalize import normalize


class Depth2RGB(object):

    """Convert depth array to rgb.

    Parameters
    ----------
    min_value: float, optional
        Minimum value for colorizing.
    max_value: float, optional
        Maximum value for colorizing.
    colormap: str, optional
        Colormap, default: 'jet'.

    """

    def __init__(self, min_value=None, max_value=None, colormap="jet"):
        self._min_value = min_value
        self._max_value = max_value
        self._colormap = colormap

    @property
    def min_value(self):
        """Minimum value of depth."""
        return self._min_value

    @property
    def max_value(self):
        """Maximum value of depth."""
        return self._max_value

    def __call__(self, depth, dtype=np.uint8):
        """Convert depth array to rgb.

        Parameters
        ----------
        depth: numpy.ndarray, (H, W), float
            Depth image.
        dtype: numpy.dtype
            Dtype of output image. default: np.uint8

        Returns
        -------
        rgb: numpy.ndarray, (H, W, 3), np.uint8
            Output colorized image.

        """
        assert depth.ndim == 2, "depth image must be 2 dimensional"
        assert np.issubdtype(depth.dtype, np.floating), "depth dtype must be float"

        normalized, self._min_value, self._max_value = normalize(
            depth,
            min_value=self._min_value,
            max_value=self._max_value,
            return_minmax=True,
        )

        isnan = np.isnan(normalized)
        normalized[isnan] = 0

        if isinstance(self._colormap, str):
            if hasattr(matplotlib, "colormaps"):
                colormap_func = matplotlib.colormaps[self._colormap]
            else:
                colormap_func = matplotlib.cm.get_cmap(self._colormap)
        else:
            colormap_func = self._colormap
        rgb = colormap_func(normalized)[:, :, :3]
        rgb[isnan] = (0, 0, 0)

        if dtype == np.uint8:
            rgb = (rgb * 255).round().astype(np.uint8)
        else:
            assert np.issubdtype(dtype, np.floating)
            rgb = rgb.astype(dtype)

        return rgb


def depth2rgb(
    depth,
    min_value=None,
    max_value=None,
    colormap="jet",
    dtype=np.uint8,
):
    # type: (np.ndarray, typing.Optional[float], typing.Optional[float], str, typing.Type) -> np.ndarray  # NOQA
    """Convert depth to rgb.

    Parameters
    ----------
    depth: numpy.ndarray, (H, W), float
        Depth image.
    dtype: numpy.dtype
        Dtype of output image. default: np.uint8
    min_value: float, optional
        Minimum value for colorizing.
    max_value: float, optional
        Maximum value for colorizing.
    colormap: str, optional
        Colormap, default: 'jet'.

    Returns
    -------
    rgb: numpy.ndarray, (H, W, 3), np.uint8
        Output colorized image.

    """
    return Depth2RGB(min_value, max_value, colormap)(depth, dtype)



import os.path as osp

import matplotlib
import PIL.Image
import PIL.ImageDraw
import PIL.ImageFont

from .. import utils


def _get_font(size, font_path=None):
    if font_path is None:
        fonts_path = osp.join(osp.dirname(matplotlib.__file__), "mpl-data/fonts/ttf")
        font_path = osp.join(fonts_path, "DejaVuSansMono.ttf")
    font = PIL.ImageFont.truetype(font=font_path, size=size)
    return font


def text_size(text, size, font_path=None):
    """Get text size (height and width).

    Parameters
    ----------
    text: str
        Text.
    size: int
        Pixel font size.

    Returns
    -------
    height: int
        Text height.
    width: int
        Text width.

    """
    font = _get_font(size, font_path=font_path)

    text_width = 0
    text_height = 0
    for line in text.splitlines():
        if line == "":
            line = "\n"

        if hasattr(font, "getbbox"):
            line_width, line_height = font.getbbox(line)[2:]
        else:
            line_width, line_height = font.getsize(line)
        text_width = max(text_width, line_width)
        text_height += line_height

    return text_height, text_width


def text(src, yx, text, size, color=(0, 0, 0), font_path=None):
    """Draw text on numpy array with Pillow.

    Parameters
    ----------
    src: numpy.ndarray
        Input image.
    yx: (2,) array-like
        Left top point of the text.
    text: str
        Text to draw.
    size: int
        Text size in pixel.
    color: (3,) array-like
        Text RGB color in uint8.
        Default is (0, 0, 0), which is black.
    font_path: str
        Default font is DejaVuSansMono in matplotlib.

    Returns
    -------
    dst: numpy.ndarray
        Output image.

    """
    dst = utils.numpy_to_pillow(src)
    text_(img=dst, yx=yx, text=text, size=size, color=color, font_path=font_path)
    return utils.pillow_to_numpy(dst)


def text_(img, yx, text, size, color=(0, 0, 0), font_path=None):
    draw = PIL.ImageDraw.ImageDraw(img)

    y1, x1 = yx
    color = tuple(color)
    font = _get_font(size=size, font_path=font_path)
    draw.text(xy=(x1, y1), text=text, fill=color, font=font)



import collections

import PIL.Image
import PIL.ImageDraw
import PIL.ImageFont

from .. import utils


def circle(src, center, diameter, fill=None, outline=None, width=0):
    """Draw circle on numpy array with Pillow.

    Parameters
    ----------
    src: numpy.ndarray
        Input image.
    center: (2,) array-like
        center is (cy, cx).
    diameter: float
        Diameter of the circle.
    fill: int or (3,) array-like, optional
        RGB color to fill the mark. None for no fill. (default: None)
    outline: int or (3,) array-like, optional
        RGB color to draw the outline.
    width: int, optional
        Rectangle line width. (default: 0)

    Returns
    -------
    dst: numpy.ndarray
        Output image.

    """
    dst = utils.numpy_to_pillow(src)
    circle_(
        img=dst,
        center=center,
        diameter=diameter,
        fill=fill,
        outline=outline,
        width=width,
    )
    return utils.pillow_to_numpy(dst)


def circle_(img, center, diameter, fill=None, outline=None, width=0):
    if isinstance(fill, collections.abc.Iterable):
        fill = tuple(fill)
    if isinstance(outline, collections.abc.Iterable):
        outline = tuple(outline)

    draw = PIL.ImageDraw.Draw(img)

    cy, cx = center

    radius = diameter / 2.0
    x1 = cx - radius
    x2 = x1 + diameter
    y1 = cy - radius
    y2 = y1 + diameter

    draw.ellipse([x1, y1, x2, y2], fill=fill, outline=outline, width=width)



# flake8: noqa

from .circle import circle
from .circle import circle_
from .ellipse import ellipse
from .ellipse import ellipse_
from .line import line
from .line import line_
from .rectangle import rectangle
from .rectangle import rectangle_
from .star import star
from .star import star_
from .text import text
from .text import text_
from .text import text_size
from .text_in_rectangle import text_in_rectangle
from .text_in_rectangle import text_in_rectangle_
from .text_in_rectangle import text_in_rectangle_aabb
from .triangle import triangle
from .triangle import triangle_



import collections

import PIL.Image
import PIL.ImageDraw
import PIL.ImageFont

from .. import utils


def ellipse(src, yx1, yx2, fill=None, outline=None, width=0):
    dst = utils.numpy_to_pillow(src)
    ellipse_(img=dst, yx1=yx1, yx2=yx2, fill=fill, outline=outline, width=width)
    return utils.pillow_to_numpy(dst)


def ellipse_(img, yx1, yx2, fill=None, outline=None, width=0):
    if isinstance(fill, collections.abc.Iterable):
        fill = tuple(fill)
    if isinstance(outline, collections.abc.Iterable):
        outline = tuple(outline)

    draw = PIL.ImageDraw.Draw(img)

    y1, x1 = yx1
    y2, x2 = yx2

    draw.ellipse([x1, y1, x2, y2], fill=fill, outline=outline, width=width)



#!/usr/bin/env python

import matplotlib.colors
import matplotlib.pyplot as plt

import imgviz


def depth2rgb():
    data = imgviz.data.arc2017()

    depthviz_jet = imgviz.depth2rgb(
        data["depth"], min_value=0.3, max_value=1, colormap="jet"
    )

    colormap = matplotlib.colors.LinearSegmentedColormap.from_list(
        "Custom", [(0, 0, 0), (0, 1, 0)]
    )
    depthviz_custom = imgviz.depth2rgb(
        data["depth"], min_value=0.3, max_value=1, colormap=colormap
    )

    # -------------------------------------------------------------------------

    plt.figure(dpi=200)

    plt.subplot(131)
    plt.title("rgb")
    plt.imshow(data["rgb"])
    plt.axis("off")

    plt.subplot(132)
    plt.title("depth (jet color)")
    plt.imshow(depthviz_jet)
    plt.axis("off")

    plt.subplot(133)
    plt.title("depth (custom color)")
    plt.imshow(depthviz_custom)
    plt.axis("off")

    img = imgviz.io.pyplot_to_numpy()
    plt.close()

    return img


if __name__ == "__main__":
    from base import run_example

    run_example(depth2rgb)



import numpy as np
import PIL.Image
import PIL.ImageDraw

from .. import utils


def line(src, yx, fill, width=1):
    dst = utils.numpy_to_pillow(src)
    line_(img=dst, yx=yx, fill=fill, width=width)
    return utils.pillow_to_numpy(dst)


def line_(img, yx, fill, width=1):
    assert isinstance(img, PIL.Image.Image)
    yx = np.asarray(yx)
    assert yx.ndim == 2
    assert yx.shape[1] == 2
    fill = tuple(fill)

    draw = PIL.ImageDraw.Draw(img)

    xy = yx[:, ::-1]
    xy = xy.flatten().tolist()
    draw.line(xy, fill=fill, width=width)



import collections

import PIL.Image

from .. import utils


def rectangle(src, aabb1, aabb2, fill=None, outline=None, width=0):
    """Draw rectangle on numpy array with Pillow.

    Parameters
    ----------
    src: numpy.ndarray
        Input image.
    aabb1: array-like, (2,)
        Minimum vertex (y_min, x_min) of the axis aligned bounding box (AABB).
    aabb2: array-like, (2,)
        Maximum vertex (y_max, x_max) of the AABB.
    fill: int or array-like, (3,), optional
        RGB color to fill the mark. None for no fill. (default: None)
    outline: int or array-like, (3,), optional
        RGB color to draw the outline.
    width: int, optional
        Rectangle line width. (default: 0)

    Returns
    -------
    dst: numpy.ndarray
        Output image.

    """
    dst = utils.numpy_to_pillow(src)
    rectangle_(
        img=dst,
        aabb1=aabb1,
        aabb2=aabb2,
        fill=fill,
        outline=outline,
        width=width,
    )
    return utils.pillow_to_numpy(dst)


def rectangle_(img, aabb1, aabb2, fill=None, outline=None, width=0):
    if isinstance(fill, collections.abc.Iterable):
        fill = tuple(fill)
    if isinstance(outline, collections.abc.Iterable):
        outline = tuple(outline)

    draw = PIL.ImageDraw.ImageDraw(img)

    y1, x1 = aabb1
    y2, x2 = aabb2
    draw.rectangle(xy=(x1, y1, x2, y2), fill=fill, outline=outline, width=width)



import numpy as np

import imgviz


def test_masks_to_bboxes():
    data = imgviz.data.arc2017()

    class_label = data["class_label"]
    masks = [class_label == label_id for label_id in np.unique(class_label)]
    bboxes = imgviz.instances.masks_to_bboxes(masks)

    assert len(bboxes) == len(masks)
    assert bboxes.shape[1] == 4

    ymin = bboxes[:, 0]
    xmin = bboxes[:, 1]
    ymax = bboxes[:, 2]
    xmax = bboxes[:, 3]
    height, width = class_label.shape
    assert ((0 <= ymin) & (ymin <= height - 1)).all()
    assert ((0 <= ymax) & (ymax <= height - 1)).all()
    assert ((0 <= xmin) & (xmin <= width - 1)).all()
    assert ((0 <= xmax) & (xmax <= width - 1)).all()



import numpy as np

from .. import color as color_module
from .. import utils
from .rectangle import rectangle_
from .text import text_
from .text import text_size


def text_in_rectangle_aabb(img_shape, loc, text, size, aabb1, aabb2, font_path=None):
    height, width = img_shape[:2]

    y1, x1 = (0, 0) if aabb1 is None else aabb1
    y2, x2 = (height - 1, width - 1) if aabb2 is None else aabb2

    tsize = text_size(text, size, font_path=font_path)

    if loc == "lt":
        yx = (y1, x1)
    elif loc == "lt+":
        yx = (y1 - tsize[0] - 2, x1)
    elif loc == "rt":
        yx = (y1, x2 - tsize[1] - 2)
    elif loc == "rt+":
        yx = (y1 - tsize[0] - 2, x2 - tsize[1] - 2)
    elif loc == "lb":
        yx = (y2 - tsize[0] - 2, 0)
    elif loc == "lb-":
        yx = (y2, 0)
    elif loc == "rb":
        yx = (y2 - tsize[0] - 2, x2 - tsize[1] - 2)
    elif loc == "rb-":
        yx = (y2, x2 - tsize[1] - 2)
    else:
        raise ValueError("unsupported loc: {}".format(loc))

    y1, x1 = yx
    y2, x2 = y1 + tsize[0] + 1, x1 + tsize[1] + 1

    return np.array([y1, x1, y2, x2])


def text_in_rectangle(
    src,
    loc,
    text,
    size,
    background,
    color=None,
    aabb1=None,
    aabb2=None,
    font_path=None,
    keep_size=False,
):
    """Draw text in a rectangle.

    Parameters
    ----------
    src: numpy.ndarray
        Input image.
    loc: str
        Location of text. It must be one of following:
        lt, rt, lb, rb, lt+, rt+, lb-, rb-.
    text: str
        Text to draw.
    size: int
        Text size in pixel.
    background: (3,) array-like
        Background color in uint8.
    color: (3,) array-like
        Text RGB color in uint8.
        If None, the color is determined by background color.
        (default: None)
    aabb1, aabb2: (2,) array-like
        Coordinate of the rectangle (y_min, x_min), (y_max, x_max).
        Default is (0, 0), (height, width).
    keep_size: bool
        Force to keep original size (size change happens with loc=xx+, xx-).

    Returns
    -------
    dst: numpy.ndarray
        Output image.

    """
    if color is None:
        color = color_module.get_fg_color(background)

    height, width = src.shape[:2]
    y1, x1, y2, x2 = text_in_rectangle_aabb(
        img_shape=src.shape,
        loc=loc,
        text=text,
        size=size,
        aabb1=aabb1,
        aabb2=aabb2,
        font_path=font_path,
    )

    dst = src

    if not keep_size:
        constant_values = (
            (background[0],),
            (background[1],),
            (background[2],),
        )
        if y1 < 0:
            pad = -y1
            dst = np.pad(
                dst,
                ((pad, 0), (0, 0), (0, 0)),
                mode="constant",
                constant_values=constant_values,
            )
            y1 += pad
            y2 += pad
        if y2 > height:
            pad = y2 - height
            dst = np.pad(
                dst,
                ((0, pad), (0, 0), (0, 0)),
                mode="constant",
                constant_values=constant_values,
            )

    dst = utils.numpy_to_pillow(dst)
    rectangle_(
        img=dst,
        aabb1=(y1, x1),
        aabb2=(y2, x2),
        fill=background,
    )
    text_(
        img=dst,
        yx=(y1 + 1, x1 + 1),
        text=text,
        color=color,
        size=size,
        font_path=font_path,
    )
    return utils.pillow_to_numpy(dst)


def text_in_rectangle_(
    img,
    loc,
    text,
    size,
    background,
    color=None,
    aabb1=None,
    aabb2=None,
    font_path=None,
):
    if color is None:
        color = color_module.get_fg_color(background)

    y1, x1, y2, x2 = text_in_rectangle_aabb(
        img_shape=(img.height, img.width),
        loc=loc,
        text=text,
        size=size,
        aabb1=aabb1,
        aabb2=aabb2,
        font_path=font_path,
    )
    rectangle_(
        img=img,
        aabb1=(y1, x1),
        aabb2=(y2, x2),
        fill=background,
    )
    text_(
        img=img,
        yx=(y1 + 1, x1 + 1),
        text=text,
        color=color,
        size=size,
        font_path=font_path,
    )
